package com.dzeio.openhealth.ui.extension

import android.app.ProgressDialog
import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.lifecycle.lifecycleScope
import androidx.navigation.fragment.navArgs
import com.dzeio.openhealth.core.BaseFragment
import com.dzeio.openhealth.databinding.FragmentExtensionBinding
import com.dzeio.openhealth.extensions.Extension
import com.dzeio.openhealth.extensions.ExtensionFactory
import dagger.hilt.android.AndroidEntryPoint
import java.lang.Exception

@AndroidEntryPoint
class ExtensionFragment :
    BaseFragment<ExtensionViewModel, FragmentExtensionBinding>(ExtensionViewModel::class.java) {

    override val bindingInflater: (LayoutInflater, ViewGroup?, Boolean) -> FragmentExtensionBinding =
        FragmentExtensionBinding::inflate

    private val args: ExtensionFragmentArgs by navArgs()

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        val extension = ExtensionFactory.getExtension(args.extension)
            ?: throw Exception("No Extension found!")
        requireActivity().actionBar?.title = extension.name

        extension.init(requireActivity())

        binding.importButton.setOnClickListener {
            val dialog = ProgressDialog(requireContext())
            dialog.setTitle("Importing...")
            dialog.setMessage("Imported 0 values")
            dialog.show()
            val data = extension.importWeight()
            data.observe(viewLifecycleOwner) { state ->
                Log.d("ExtensionFragment", state.state.name)
                Log.d("ExtensionFragment", state.list.size.toString())
                dialog.setMessage("Imported ${state.list.size} values")
                if (state.state == Extension.States.DONE) {
                    dialog.setMessage("Finishing Import...")
                    lifecycleScope.launchWhenStarted {
                        state.list.forEach {
                            it.source = extension.id
                            viewModel.importWeight(it)
                        }
                        dialog.dismiss()
                    }
                }
            }
        }
    }
}
