package com.dzeio.openhealth.ui.extensions

import android.content.Intent
import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Toast
import androidx.activity.result.contract.ActivityResultContracts
import androidx.navigation.fragment.findNavController
import androidx.recyclerview.widget.LinearLayoutManager
import com.dzeio.openhealth.R
import com.dzeio.openhealth.adapters.ExtensionAdapter
import com.dzeio.openhealth.core.BaseFragment
import com.dzeio.openhealth.databinding.FragmentExtensionsBinding
import com.dzeio.openhealth.extensions.Extension
import com.dzeio.openhealth.extensions.GoogleFit
import com.dzeio.openhealth.utils.PermissionsManager
import dagger.hilt.android.AndroidEntryPoint

@AndroidEntryPoint
class ExtensionsFragment :
    BaseFragment<ExtensionsViewModel, FragmentExtensionsBinding>(ExtensionsViewModel::class.java) {

    companion object {
        const val TAG = "ExtensionsFragment"
    }

    override val bindingInflater: (LayoutInflater, ViewGroup?, Boolean) -> FragmentExtensionsBinding =
        FragmentExtensionsBinding::inflate

    private lateinit var activeExtension: Extension

    private val activityResult = registerForActivityResult(
        ActivityResultContracts.RequestMultiplePermissions()
    ) { map ->
        if (map.containsValue(false)) {
            // TODO: Show a popup with choice to change it
            Toast.makeText(requireContext(), R.string.permission_declined, Toast.LENGTH_LONG).show()
            return@registerForActivityResult
        }

        extensionIsConnected(activeExtension)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        val recycler = binding.list

        val manager = LinearLayoutManager(requireContext())
        recycler.layoutManager = manager

        val adapter = ExtensionAdapter()
        adapter.onItemClick = {
            activeExtension = it
            Log.d(TAG, "${it.id}: ${it.name}")

            this.extensionPermissionsVerified(it)
        }
        recycler.adapter = adapter

        val list = arrayOf(
            GoogleFit()
        ).toList()

        list.forEach {
            it.init(requireActivity())
        }

        adapter.set(list)
    }

    @Deprecated("Deprecated in Java")
    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        activeExtension.onActivityResult(requestCode, resultCode, data)
    }

    private fun extensionPermissionsVerified(it: Extension) {
        // Check for the extension permissions
        if (it.permissions != null && !PermissionsManager.hasPermission(
                requireContext(),
                it.permissions!!
            )
        ) {
            // TODO: show popup explaining the permissions requested

            // show permissions
            activityResult.launch(it.permissions)
            return
        }
        extensionIsConnected(it)
    }

    private fun extensionIsConnected(it: Extension) {
        // check if it is connected
        if (it.isConnected()) {
            gotoExtension(it)
            return
        }

        // IDK: maybe give less liberty to the extension IDK
        val ld = it.connect()
        ld.observe(viewLifecycleOwner) { state ->
            Log.d(TAG, state.toString())
            if (state == Extension.States.DONE) {
                gotoExtension(it)
            }
        }
    }

    private fun gotoExtension(it: Extension) {
        findNavController().navigate(
            ExtensionsFragmentDirections.actionNavExtensionsToNavExtension(it.id)
        )
    }
}
