package com.dzeio.openhealth.ui.water

import android.view.LayoutInflater
import androidx.core.view.marginBottom
import com.dzeio.openhealth.R
import com.dzeio.openhealth.core.BaseDialog
import com.dzeio.openhealth.databinding.DialogWaterSizeSelectorBinding
import com.google.android.material.dialog.MaterialAlertDialogBuilder
import com.google.android.material.textfield.TextInputEditText
import com.google.android.material.textfield.TextInputLayout
import dagger.hilt.android.AndroidEntryPoint

@AndroidEntryPoint
class WaterSizeSelectorDialog :
    BaseDialog<WaterSizeSelectorViewModel, DialogWaterSizeSelectorBinding>(
        WaterSizeSelectorViewModel::class.java
    ) {

    override val bindingInflater: (LayoutInflater) -> DialogWaterSizeSelectorBinding
        get() = DialogWaterSizeSelectorBinding::inflate

    override fun onCreated() {
        super.onCreated()

        binding.cancel.setOnClickListener {
            dismiss()
        }

        binding.validate.setOnClickListener {
            dismiss()
        }

        viewModel.cupSize.observe(this) {
            binding.customSize.text = String.format(
                getString(R.string.custom_amount),
                "${it}ml"
            )
        }

        binding.size100ml.setOnClickListener {
            viewModel.setCupSize(100)
        }
        binding.size250ml.setOnClickListener {
            viewModel.setCupSize(250)
        }
        binding.size500ml.setOnClickListener {
            viewModel.setCupSize(500)
        }
        binding.size1000ml.setOnClickListener {
            viewModel.setCupSize(1000)
        }

        binding.customSize.setOnClickListener {
            val editTextLayout = TextInputLayout(requireContext())

            val editText = TextInputEditText(requireContext())
            editText.setText(viewModel.cupSize.value.toString())

            editTextLayout.addView(editText)

            MaterialAlertDialogBuilder(requireContext())
                .setView(editTextLayout)
                .setTitle("Custom Cup Size")
                .setOnCancelListener {
                    it.dismiss()
                }
                .setNegativeButton(R.string.cancel) { dialog, _ ->
                    dialog.dismiss()
                }
                .setPositiveButton(
                    R.string.validate
                ) { dialog, _ ->
                    viewModel.setCupSize(editText.text.toString().toInt())
                    dismiss()
                    dialog.dismiss()
                }
                .show()
        }
    }
}
